# coding=utf-8

"""
@fileName       :   __init__.py
@data           :   2024/2/3
@author         :   jiangmenggui@hosonsoft.com
"""
from .attribute_dict.model import AttributeDict, IgnoreCaseAttributeDict

if __name__ == '__main__':
    pass
